package com.darcy.flutterUUID.flutter_gm_uuid

import android.app.Activity
import android.content.Context
import android.content.Context.TELEPHONY_SERVICE
import android.os.Build
import android.provider.Settings
import android.telephony.TelephonyManager
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.util.*

/** FlutterGmUuidPlugin */
public class FlutterGmUuidPlugin : FlutterPlugin, MethodCallHandler ,ActivityAware{
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel

    private lateinit var context: Context
    private lateinit var activity: Activity


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "UUID_channel")
        channel.setMethodCallHandler(this);
        context=flutterPluginBinding.applicationContext
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object
    {

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "UUID_channel")
            channel.setMethodCallHandler(FlutterGmUuidPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "getUuid") 
        {
            var deviceIdQ: String? = null

            //大于P版采用mac地址储存。
            if (Build.VERSION.SDK_INT >  Build.VERSION_CODES.P) 
            {
                //todo android Q 处理
                //先用网上采用的mac地址，弄不到，就搞个UUID垃圾吧到
                //https://blog.csdn.net/dqliangjun/article/details/79120776

                try {
                    //获取设备的MACAddress地址 去掉中间相隔的冒号
                    //   deviceIdQ = getLocalMac(mActivity);
                    //2019-07-22 Q版本的唯一识别为SSAID  生成是基于设备用户安装应用签名的密钥
                    //   暂时先这么处理
                    // deviceIdQ = Settings.Secure.ANDROID_ID; //android_id,此方案不行，
                    //  Google给出的解决方案是：如果您的应用有 追踪非登录用户重装 的需求，可用ANDROID_ID来标识设备。

                    //  ANDROID_ID的生成规则为：签名+设备信息+设备用户
                    //  ANDROID_ID重置规则：设备恢复出厂设置时，ANDROID_ID将被重置
                    deviceIdQ = Settings.Secure.getString(activity.getContentResolver(), Settings.Secure.ANDROID_ID)
                    if (TextUtils.isEmpty(deviceIdQ)) {
                        deviceIdQ = getLocalMac()?.replace(":", "")
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                //如果以上搜没有获取相应的则自己生成相应的UUID作为相应设备唯一标识符
                if (TextUtils.isEmpty(deviceIdQ)) {
                    val uuid = UUID.randomUUID()
                    deviceIdQ = uuid.toString().replace("-", "")
                }
              result.success(deviceIdQ)
            } 
            else 
            {
                
                val telephonyManager = activity.getSystemService(TELEPHONY_SERVICE) as TelephonyManager
                deviceIdQ = telephonyManager.deviceId
                Log.d("有没有走到这里",deviceIdQ)
                result.success(deviceIdQ)
                
            }
            
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onDetachedFromActivity() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity=binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }


}
