package com.darcy.flutterUUID.flutter_gm_uuid

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.os.Build
import android.provider.Settings
import android.telephony.TelephonyManager
import android.text.TextUtils
import android.widget.Toast
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.util.*

/**
 * Author: Shay-Patrick-Cormac
 * Email: fang47881@126.com
 * Ltd: 金螳螂企业（集团）有限公司
 * Date: 2020/6/15 16:57
 * Version: 1.0
 * Description: 类说明
 */
class PluginMethodCallHandler(private val activity: Activity):MethodChannel.MethodCallHandler {
    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {

        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "getUuid") {
            var deviceIdQ: String? = null

            //大于P版采用mac地址储存。
            if (Build.VERSION.SDK_INT > Build.VERSION_CODES.P) {
                //todo android Q 处理
                //先用网上采用的mac地址，弄不到，就搞个UUID垃圾吧到
                //https://blog.csdn.net/dqliangjun/article/details/79120776

                try {
                    //获取设备的MACAddress地址 去掉中间相隔的冒号
                    //   deviceIdQ = getLocalMac(mActivity);
                    //2019-07-22 Q版本的唯一识别为SSAID  生成是基于设备用户安装应用签名的密钥
                    //   暂时先这么处理
                    // deviceIdQ = Settings.Secure.ANDROID_ID; //android_id,此方案不行，
                    //  Google给出的解决方案是：如果您的应用有 追踪非登录用户重装 的需求，可用ANDROID_ID来标识设备。

                    //  ANDROID_ID的生成规则为：签名+设备信息+设备用户
                    //  ANDROID_ID重置规则：设备恢复出厂设置时，ANDROID_ID将被重置
                    deviceIdQ = Settings.Secure.getString(activity.getContentResolver(), Settings.Secure.ANDROID_ID)
                    if (TextUtils.isEmpty(deviceIdQ)) {
                        deviceIdQ = getLocalMac()?.replace(":", "")
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                //如果以上搜没有获取相应的则自己生成相应的UUID作为相应设备唯一标识符
                if (TextUtils.isEmpty(deviceIdQ)) {
                    val uuid = UUID.randomUUID()
                    deviceIdQ = uuid.toString().replace("-", "")
                }
                result.success(deviceIdQ)
            } else {
                CompatPermissionManager.INSTANCE.with(activity).tag(0x00000008).permission(PermissionEnum.READ_PHONE_STATE).callback(object : PermissionsCallback {
                    //imei唯一标志码
                    @SuppressLint("MissingPermission")
                    override fun onGranted(grantedList: ArrayList<PermissionEnum?>?) {
                        //权限申请
                        val telephonyManager = activity.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
                        deviceIdQ = telephonyManager.deviceId
                        result.success(deviceIdQ)

                    }

                    override fun onDenied(deniedList: ArrayList<PermissionEnum>?) {
                        Toast.makeText(activity, "权限拒绝了", Toast.LENGTH_LONG).show()
                        result.error("500", "未授予权限", null)
                    }

                }).checkAsk()

            }

        } else {
            result.notImplemented()
        }
    }
}